import logging
from datetime import datetime

import pytz
from sqlalchemy import delete
from sqlalchemy import distinct
from sqlalchemy import select
from txcelery.defer import DeferrableTask

from peek_core_docdb._private.storage.DocDbCompilerQueue import (
    DocDbCompilerQueue,
)
from peek_core_docdb._private.storage.DocDbDocument import DocDbDocument
from peek_core_docdb._private.worker.tasks.ImportTask import _loadModelSets
from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_base.worker.CeleryApp import celeryApp


logger = logging.getLogger(__name__)


# We need to insert the into the following tables:


@DeferrableTask
@celeryApp.task(bind=True)
def deleteDocumentsForImportGropHashTask(
    self, modelSetKey: str, importGroupHash: str
) -> None:
    startTime = datetime.now(pytz.utc)

    modelSetIdByKey = _loadModelSets()

    # Do the import
    try:
        modelSetId = modelSetIdByKey.get(modelSetKey)
        _deleteObjectsAndQueueChunkCompile(modelSetId, importGroupHash)

        logger.info(
            f"Deleted Documents for"
            f" modelSetKey={modelSetKey},"
            f" importGroupHash='{importGroupHash}',"
            f" in {datetime.now(pytz.utc) - startTime}"
        )

    except Exception as e:
        logger.debug("Retrying import docDb objects, %s", e)
        raise self.retry(exc=e, countdown=3)


def _deleteObjectsAndQueueChunkCompile(
    modelSetId: int, importGroupHash: str
) -> None:
    """Insert or Update Objects

    1) Get the chunks we need to recompile
    2) Delete the required objects
    3) Queue the chunks for recompile

    """

    documentTable = DocDbDocument.__table__
    queueTable = DocDbCompilerQueue.__table__

    startTime = datetime.now(pytz.utc)

    engine = CeleryDbConn.getDbEngine()
    conn = engine.connect()
    transaction = conn.begin()

    try:

        chunkKeyQuery = select(distinct(documentTable.c.chunkKey)).where(
            documentTable.c.importGroupHash == importGroupHash,
            documentTable.c.modelSetId == modelSetId,
        )

        chunkKeysToQueue = conn.execute(chunkKeyQuery).scalars().all()

        deleteStmt = delete(DocDbDocument).where(
            documentTable.c.importGroupHash == importGroupHash,
            documentTable.c.modelSetId == modelSetId,
        )

        result = conn.execute(deleteStmt)
        deletedDocCount = result.rowcount

        if chunkKeysToQueue:
            conn.execute(
                queueTable.insert(),
                [
                    dict(modelSetId=modelSetId, chunkKey=c)
                    for c in chunkKeysToQueue
                ],
            )

        if deletedDocCount or chunkKeysToQueue:
            transaction.commit()
        else:
            transaction.rollback()

        logger.debug(
            "Deleted %s, queued %s chunks in %s",
            deletedDocCount,
            len(chunkKeysToQueue),
            (datetime.now(pytz.utc) - startTime),
        )

    except Exception:
        transaction.rollback()
        raise

    finally:
        conn.close()



from typing import List

from twisted.internet.defer import Deferred

from peek_core_docdb._private.server.controller.ImportController import (
    ImportController,
)
from peek_core_docdb.server.DocDbApiABC import DocDbApiABC


class DocDbApi(DocDbApiABC):
    def __init__(self, importController: ImportController):
        self._importController = importController

    def shutdown(self):
        pass

    def createOrUpdateDocuments(
        self, documentsEncodedPayload: bytes
    ) -> Deferred:
        return self._importController.createOrUpdateDocuments(
            documentsEncodedPayload
        )

    def deleteDocuments(self, modelSetKey: str, keys: List[str]) -> Deferred:
        pass

    def deleteDocumentsForImportGropHash(
        self, modelSetKey: str, importGroupHash: str
    ) -> Deferred:
        return self._importController.deleteDocumentsForImportGropHash(
            modelSetKey, importGroupHash
        )



import json
import logging
from collections import defaultdict
from typing import List
from typing import Union

from peek_core_docdb._private.client.controller.DocumentCacheController import (
    DocumentCacheController,
)
from peek_core_docdb._private.storage.DocDbDocumentTypeTuple import (
    DocDbDocumentTypeTuple,
)
from peek_core_docdb._private.storage.DocDbEncodedChunk import DocDbEncodedChunk
from peek_core_docdb._private.storage.DocDbModelSet import DocDbModelSet
from peek_core_docdb._private.worker.tasks._CalcChunkKey import makeChunkKey
from peek_core_docdb.tuples.DocumentTuple import DocumentTuple
from twisted.internet.defer import Deferred

from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Jsonable import Jsonable
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class ClientDocumentTupleProvider(TuplesProviderABC):
    def __init__(self, cacheHandler: DocumentCacheController):
        self._cacheHandler = cacheHandler

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        modelSetKey = tupleSelector.selector["modelSetKey"]
        keys = tupleSelector.selector["keys"]

        keysByChunkKey = defaultdict(list)

        foundDocuments: List[DocumentTuple] = []

        for key in keys:
            keysByChunkKey[makeChunkKey(modelSetKey, key)].append(key)

        for chunkKey, subKeys in keysByChunkKey.items():
            # JS Side = PrivateDocumentLoaderService.getDocumentsForKeys
            chunkData: DocDbEncodedChunk = self._cacheHandler.encodedChunk(
                chunkKey
            )

            if not chunkData:
                logger.warning(
                    "Document chunk %s is missing from cache", chunkKey
                )
                continue

            docsByKeyStr = (
                Payload().fromEncodedPayload(chunkData.encodedData).tuples[0]
            )
            docsByKey = json.loads(docsByKeyStr)

            for subKey in subKeys:
                if subKey not in docsByKey:
                    logger.warning(
                        "Document %s is missing from index, chunkKey %s",
                        subKey,
                        chunkKey,
                    )
                    continue

                # Reconstruct the data
                objectProps: {} = json.loads(docsByKey[subKey])
                objectProps = Jsonable().fromJsonField(objectProps)

                # Get out the object type
                thisDocumentTypeId = objectProps["_dtid"]
                del objectProps["_dtid"]

                # Get out the object type
                thisModelSetId = objectProps["_msid"]
                del objectProps["_msid"]

                # Create the new object
                newObject = DocumentTuple()
                foundDocuments.append(newObject)

                newObject.key = subKey
                newObject.modelSet = DocDbModelSet(id=thisModelSetId)
                newObject.documentType = DocDbDocumentTypeTuple(
                    id=thisDocumentTypeId
                )
                newObject.document = objectProps

        # Create the vortex message
        return (
            Payload(filt, tuples=foundDocuments)
            .makePayloadEnvelope()
            .toVortexMsg()
        )



from peek_core_docdb._private.PluginNames import docDbTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple


@addTupleType
class AdminStatusTuple(Tuple):
    __tupleType__ = docDbTuplePrefix + "AdminStatusTuple"

    documentCompilerQueueStatus: bool = TupleField(False)
    documentCompilerQueueSize: int = TupleField(0)
    documentCompilerQueueProcessedTotal: int = TupleField(0)
    documentCompilerQueueLastError: str = TupleField()



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_core_docdb._private.PluginNames import docDbFilt
from peek_core_docdb._private.PluginNames import docDbActionProcessorName
from vortex.handler.TupleActionProcessorProxy import TupleActionProcessorProxy


def makeTupleActionProcessorProxy():
    return TupleActionProcessorProxy(
        tupleActionProcessorName=docDbActionProcessorName,
        proxyToVortexName=peekServerName,
        additionalFilt=docDbFilt,
    )



"""Change to BigInteger

Peek Plugin Database Migration Script

Revision ID: 0b20ccfb7a6a
Revises: 6c94c040e18c
Create Date: 2020-05-01 23:59:15.448423

"""

# revision identifiers, used by Alembic.
revision = "0b20ccfb7a6a"
down_revision = "6c94c040e18c"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy import text
import geoalchemy2


def __alterColumnPkBigInt(schemaName, tableName):
    return text(
        """
        
        DO $$
            DECLARE
                rec RECORD;
            BEGIN
            FOR rec IN SELECT
                        tc.table_schema, 
                        tc.table_name, 
                        kcu.column_name
                    FROM 
                        information_schema.table_constraints AS tc 
                        JOIN information_schema.key_column_usage AS kcu
                          ON tc.constraint_name = kcu.constraint_name
                          AND tc.table_schema = kcu.table_schema
                        JOIN information_schema.constraint_column_usage AS ccu
                          ON ccu.constraint_name = tc.constraint_name
                          AND ccu.table_schema = tc.table_schema
                    WHERE tc.constraint_type = 'FOREIGN KEY'
                        AND tc.table_schema='%(schemaName)s'
                        AND ccu.table_name='%(tableName)s'
                        AND ccu.column_name='id'
            LOOP 
                EXECUTE 'ALTER TABLE "' || rec.table_schema || '"'
                        ||'."' || rec.table_name || '" '
                        || ' ALTER COLUMN "' || rec.column_name || '" TYPE bigint';
            END LOOP;
            END
        $$;
        
        ALTER TABLE %(schemaName)s."%(tableName)s" ALTER COLUMN id TYPE bigint;
        
        DROP SEQUENCE IF EXISTS %(schemaName)s."%(tableName)s_id_seq" CASCADE;

        CREATE SEQUENCE %(schemaName)s."%(tableName)s_id_seq" AS bigint
            INCREMENT 1
            START 1
            MINVALUE 0
            MAXVALUE 9223372036854775807
            CACHE 1000
            OWNED BY "%(schemaName)s"."%(tableName)s"."%(columnName)s";
    
        SELECT setval('%(schemaName)s."%(tableName)s_%(columnName)s_seq"', 
                max("%(columnName)s") + 1, FALSE)
        FROM %(schemaName)s."%(tableName)s";
        
        ALTER TABLE %(schemaName)s."%(tableName)s"
        ALTER COLUMN "%(columnName)s" SET DEFAULT 
            nextval('%(schemaName)s."%(tableName)s_%(columnName)s_seq"'::regclass);
        """
        % dict(schemaName=schemaName, tableName=tableName, columnName="id")
    )


def upgrade():
    op.execute(__alterColumnPkBigInt("core_docdb", "DocDbChunkQueue"))
    op.execute(__alterColumnPkBigInt("core_docdb", "DocDbDocument"))
    op.execute(__alterColumnPkBigInt("core_docdb", "DocDbEncodedChunkTuple"))


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



from peek_core_docdb._private.client.controller.DocumentCacheController import (
    DocumentCacheController,
)
from peek_core_docdb._private.client.tuple_providers.ClientDocumentTupleProvider import (
    ClientDocumentTupleProvider,
)
from peek_core_docdb._private.client.tuple_providers.ClientDocumentUpdateDateTupleProvider import (
    ClientDocumentUpdateDateTupleProvider,
)
from peek_core_docdb._private.tuples.DocumentUpdateDateTuple import (
    DocumentUpdateDateTuple,
)
from peek_core_docdb.tuples.DocumentTuple import DocumentTuple
from vortex.handler.TupleDataObservableProxyHandler import (
    TupleDataObservableProxyHandler,
)


def makeClientTupleDataObservableHandler(
    tupleObservable: TupleDataObservableProxyHandler,
    cacheHandler: DocumentCacheController,
):
    """ " Make CLIENT Tuple Data Observable Handler

    This method registers the tuple providers for the proxy, that are served locally.

    :param cacheHandler:
    :param tupleObservable: The tuple observable proxy

    """

    tupleObservable.addTupleProvider(
        DocumentTuple.tupleName(), ClientDocumentTupleProvider(cacheHandler)
    )

    tupleObservable.addTupleProvider(
        DocumentUpdateDateTuple.tupleName(),
        ClientDocumentUpdateDateTupleProvider(cacheHandler),
    )



from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Index
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import orm
from sqlalchemy.orm import relationship
from vortex.Tuple import Tuple
from vortex.Tuple import addTupleType

from peek_core_docdb._private.PluginNames import docDbTuplePrefix
from peek_core_docdb._private.storage.DeclarativeBase import DeclarativeBase
from peek_core_docdb._private.storage.DocDbModelSet import DocDbModelSet


@addTupleType
class DocDbDocumentTypeTuple(DeclarativeBase, Tuple):
    __tupleType__ = docDbTuplePrefix + "DocDbDocumentTypeTuple"
    __tablename__ = "DocDbDocumentType"

    id = Column(Integer, primary_key=True, autoincrement=True)

    #:  The model set for this document
    modelSetId = Column(
        Integer,
        ForeignKey("DocDbModelSet.id", ondelete="CASCADE"),
        nullable=False,
    )
    modelSet = relationship(DocDbModelSet)

    name = Column(String, nullable=False)
    title = Column(String, nullable=False)

    __table_args__ = (
        Index("idx_DocType_model_name", modelSetId, name, unique=True),
        Index("idx_DocType_model_title", modelSetId, title, unique=True),
    )

    @orm.reconstructor
    def __init__(self, **kwargs):
        Tuple.__init__(self, **kwargs)



import logging

from peek_abstract_chunked_index.private.server.controller.ACIProcessorQueueControllerABC import (
    ACIProcessorQueueControllerABC,
    ACIProcessorQueueBlockItem,
)
from peek_abstract_chunked_index.private.server.controller.ACIProcessorStatusNotifierABC import (
    ACIProcessorStatusNotifierABC,
)
from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_core_docdb._private.server.client_handlers.ClientChunkUpdateHandler import (
    ClientChunkUpdateHandler,
)
from peek_core_docdb._private.server.controller.StatusController import (
    StatusController,
)
from peek_core_docdb._private.storage.DocDbCompilerQueue import (
    DocDbCompilerQueue,
)
from peek_core_docdb._private.storage.DocDbDocument import DocDbDocument
from peek_core_docdb._private.storage.DocDbEncodedChunk import DocDbEncodedChunk

logger = logging.getLogger(__name__)


class _Notifier(ACIProcessorStatusNotifierABC):
    def __init__(self, adminStatusController: StatusController):
        self._adminStatusController = adminStatusController

    def setProcessorStatus(self, state: bool, queueSize: int):
        self._adminStatusController.status.documentCompilerQueueStatus = state
        self._adminStatusController.status.documentCompilerQueueSize = queueSize
        self._adminStatusController.notify()

    def addToProcessorTotal(self, delta: int):
        self._adminStatusController.status.documentCompilerQueueProcessedTotal += (
            delta
        )
        self._adminStatusController.notify()

    def setProcessorError(self, error: str):
        self._adminStatusController.status.documentCompilerQueueLastError = (
            error
        )
        self._adminStatusController.notify()


class ChunkCompilerQueueController(ACIProcessorQueueControllerABC):
    """DocDbChunkCompilerQueueController

    Compile the disp items into the grid data

    1) Query for queue
    2) Process queue
    3) Delete from queue

    """

    QUEUE_ITEMS_PER_TASK = 10
    POLL_PERIOD_SECONDS = 1.000

    QUEUE_BLOCKS_MAX = 20
    QUEUE_BLOCKS_MIN = 4

    WORKER_TASK_TIMEOUT = 60.0

    _logger = logger
    _QueueDeclarative: ACIProcessorQueueTupleABC = DocDbCompilerQueue
    _VacuumDeclaratives = (DocDbCompilerQueue, DocDbDocument, DocDbEncodedChunk)

    def __init__(
        self,
        dbSessionCreator,
        statusController: StatusController,
        clientChunkUpdateHandler: ClientChunkUpdateHandler,
    ):
        ACIProcessorQueueControllerABC.__init__(
            self, dbSessionCreator, _Notifier(statusController)
        )

        self._clientChunkUpdateHandler: ClientChunkUpdateHandler = (
            clientChunkUpdateHandler
        )

    def _sendToWorker(self, block: ACIProcessorQueueBlockItem):
        from peek_core_docdb._private.worker.tasks.ChunkCompilerTask import (
            compileDocumentChunk,
        )

        return compileDocumentChunk.delay(block.itemsEncodedPayload)

    def _processWorkerResults(self, results):
        self._clientChunkUpdateHandler.sendChunks(results)

    def _dedupeQueueSql(self, lastFetchedId: int, dedupeLimit: int):
        return """
                 with sq_raw as (
                    SELECT "id", "chunkKey"
                    FROM core_docdb."DocDbChunkQueue"
                    WHERE id > %(id)s
                    LIMIT %(limit)s
                ), sq as (
                    SELECT min(id) as "minId", "chunkKey"
                    FROM sq_raw
                    GROUP BY  "chunkKey"
                    HAVING count("chunkKey") > 1
                )
                DELETE
                FROM core_docdb."DocDbChunkQueue"
                     USING sq sq1
                WHERE core_docdb."DocDbChunkQueue"."id" != sq1."minId"
                    AND core_docdb."DocDbChunkQueue"."id" > %(id)s
                    AND core_docdb."DocDbChunkQueue"."chunkKey" = sq1."chunkKey"

            """ % {
            "id": lastFetchedId,
            "limit": dedupeLimit,
        }



from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import orm
from vortex.Tuple import Tuple
from vortex.Tuple import addTupleType

from peek_core_docdb._private.PluginNames import docDbTuplePrefix
from .DeclarativeBase import DeclarativeBase


@addTupleType
class DocDbModelSet(DeclarativeBase, Tuple):
    __tablename__ = "DocDbModelSet"
    __tupleType__ = docDbTuplePrefix + __tablename__

    id = Column(Integer, primary_key=True, autoincrement=True)
    key = Column(String, nullable=False, unique=True)
    name = Column(String, nullable=False, unique=True)

    comment = Column(String)
    propsJson = Column(String)

    @orm.reconstructor
    def __init__(self, **kwargs):
        Tuple.__init__(self, **kwargs)



import logging

from sqlalchemy import Column, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_core_docdb._private.PluginNames import docDbTuplePrefix
from .DeclarativeBase import DeclarativeBase

logger = logging.getLogger(__name__)


@addTupleType
class DocDbCompilerQueue(Tuple, DeclarativeBase, ACIProcessorQueueTupleABC):
    __tupleType__ = docDbTuplePrefix + "DocDbChunkQueueTuple"
    __tablename__ = "DocDbChunkQueue"

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    modelSetId = Column(
        Integer,
        ForeignKey("DocDbModelSet.id", ondelete="CASCADE"),
        nullable=False,
    )

    chunkKey = Column(String, primary_key=True)

    @classmethod
    def sqlCoreLoad(cls, row):
        return DocDbCompilerQueue(
            id=row.id, modelSetId=row.modelSetId, chunkKey=row.chunkKey
        )

    @property
    def ckiUniqueKey(self):
        return self.chunkKey



import logging

from vortex.sqla_orm.OrmCrudHandler import OrmCrudHandler

from peek_core_docdb._private.PluginNames import docDbFilt
from peek_core_docdb._private.storage.DocDbDocument import DocDbDocument

logger = logging.getLogger(__name__)

# This dict matches the definition in the Admin angular app.
filtKey = {"key": "admin.Edit.DocumentTuple"}
filtKey.update(docDbFilt)


# This is the CRUD hander
class __CrudHandler(OrmCrudHandler):
    def createDeclarative(self, session, payloadFilt):
        return (
            session.query(DocDbDocument)
            .filter(DocDbDocument.key == payloadFilt.get("docKey"))
            .all()
        )


# This method creates an instance of the handler class.
def makeDocumentTableHandler(tupleObservable, dbSessionCreator):
    handler = __CrudHandler(dbSessionCreator, DocDbDocument, filtKey)

    logger.debug("Started")
    return handler



"""initial tables

Peek Plugin Database Migration Script

Revision ID: c1d2d5475c64
Revises: 
Create Date: 2018-06-21 22:49:45.976813

"""

# revision identifiers, used by Alembic.
revision = "c1d2d5475c64"
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "DocDbModelSet",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("propsJson", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("key"),
        sa.UniqueConstraint("name"),
        schema="core_docdb",
    )
    op.create_table(
        "Setting",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_table(
        "DocDbChunkQueue",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["core_docdb.DocDbModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="core_docdb",
    )
    op.create_table(
        "DocDbDocumentType",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("title", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["core_docdb.DocDbModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_index(
        "idx_DocType_model_name",
        "DocDbDocumentType",
        ["modelSetId", "name"],
        unique=True,
        schema="core_docdb",
    )
    op.create_index(
        "idx_DocType_model_title",
        "DocDbDocumentType",
        ["modelSetId", "title"],
        unique=True,
        schema="core_docdb",
    )
    op.create_table(
        "DocDbEncodedChunkTuple",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.Column("encodedData", sa.LargeBinary(), nullable=False),
        sa.Column("encodedHash", sa.String(), nullable=False),
        sa.Column("lastUpdate", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["core_docdb.DocDbModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_index(
        "idx_Chunk_modelSetId_chunkKey",
        "DocDbEncodedChunkTuple",
        ["modelSetId", "chunkKey"],
        unique=False,
        schema="core_docdb",
    )
    op.create_table(
        "DocDbProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("title", sa.String(), nullable=False),
        sa.Column("order", sa.Integer(), server_default="0", nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["core_docdb.DocDbModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_index(
        "idx_DocDbProp_model_name",
        "DocDbProperty",
        ["modelSetId", "name"],
        unique=True,
        schema="core_docdb",
    )
    op.create_index(
        "idx_DocDbProp_model_title",
        "DocDbProperty",
        ["modelSetId", "title"],
        unique=True,
        schema="core_docdb",
    )
    op.create_table(
        "SettingProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("settingId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=16), nullable=True),
        sa.Column("int_value", sa.Integer(), nullable=True),
        sa.Column("char_value", sa.String(), nullable=True),
        sa.Column("boolean_value", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(
            ["settingId"],
            ["core_docdb.Setting.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_index(
        "idx_SettingProperty_settingId",
        "SettingProperty",
        ["settingId"],
        unique=False,
        schema="core_docdb",
    )
    op.create_table(
        "DocDbDocument",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("documentTypeId", sa.Integer(), nullable=False),
        sa.Column("importGroupHash", sa.String(), nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("chunkKey", sa.String(), nullable=False),
        sa.Column("documentJson", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["documentTypeId"],
            ["core_docdb.DocDbDocumentType.id"],
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["core_docdb.DocDbModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_docdb",
    )
    op.create_index(
        "idx_Document_documentType",
        "DocDbDocument",
        ["documentTypeId"],
        unique=False,
        schema="core_docdb",
    )
    op.create_index(
        "idx_Document_gridKey",
        "DocDbDocument",
        ["chunkKey"],
        unique=False,
        schema="core_docdb",
    )
    op.create_index(
        "idx_Document_importGroupHash",
        "DocDbDocument",
        ["importGroupHash"],
        unique=False,
        schema="core_docdb",
    )
    op.create_index(
        "idx_Document_key",
        "DocDbDocument",
        ["modelSetId", "key"],
        unique=True,
        schema="core_docdb",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_Document_key", table_name="DocDbDocument", schema="core_docdb"
    )
    op.drop_index(
        "idx_Document_importGroupHash",
        table_name="DocDbDocument",
        schema="core_docdb",
    )
    op.drop_index(
        "idx_Document_gridKey", table_name="DocDbDocument", schema="core_docdb"
    )
    op.drop_index(
        "idx_Document_documentType",
        table_name="DocDbDocument",
        schema="core_docdb",
    )
    op.drop_table("DocDbDocument", schema="core_docdb")
    op.drop_index(
        "idx_SettingProperty_settingId",
        table_name="SettingProperty",
        schema="core_docdb",
    )
    op.drop_table("SettingProperty", schema="core_docdb")
    op.drop_index(
        "idx_DocDbProp_model_title",
        table_name="DocDbProperty",
        schema="core_docdb",
    )
    op.drop_index(
        "idx_DocDbProp_model_name",
        table_name="DocDbProperty",
        schema="core_docdb",
    )
    op.drop_table("DocDbProperty", schema="core_docdb")
    op.drop_index(
        "idx_Chunk_modelSetId_chunkKey",
        table_name="DocDbEncodedChunkTuple",
        schema="core_docdb",
    )
    op.drop_table("DocDbEncodedChunkTuple", schema="core_docdb")
    op.drop_index(
        "idx_DocType_model_title",
        table_name="DocDbDocumentType",
        schema="core_docdb",
    )
    op.drop_index(
        "idx_DocType_model_name",
        table_name="DocDbDocumentType",
        schema="core_docdb",
    )
    op.drop_table("DocDbDocumentType", schema="core_docdb")
    op.drop_table("DocDbChunkQueue", schema="core_docdb")
    op.drop_table("Setting", schema="core_docdb")
    op.drop_table("DocDbModelSet", schema="core_docdb")
    # ### end Alembic commands ###



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_core_docdb._private.PluginNames import docDbFilt
from peek_core_docdb._private.PluginNames import docDbObservableName
from vortex.handler.TupleDataObservableProxyHandler import (
    TupleDataObservableProxyHandler,
)


def makeDeviceTupleDataObservableProxy():
    return TupleDataObservableProxyHandler(
        observableName=docDbObservableName,
        proxyToVortexName=peekServerName,
        additionalFilt=docDbFilt,
    )



import logging
from typing import Union

from twisted.internet.defer import Deferred

from peek_core_docdb._private.storage.DocDbDocumentTypeTuple import (
    DocDbDocumentTypeTuple,
)
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class DocumentTypeTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        session = self._ormSessionCreator()
        try:
            tuples = session.query(DocDbDocumentTypeTuple).all()

            # Create the vortex message
            return (
                Payload(filt, tuples=tuples).makePayloadEnvelope().toVortexMsg()
            )

        finally:
            session.close()



"""Added showOn fields

Peek Plugin Database Migration Script

Revision ID: 931b88db3117
Revises: a423a783700f
Create Date: 2019-07-28 20:15:55.922299

"""

# revision identifiers, used by Alembic.
revision = "931b88db3117"
down_revision = "a423a783700f"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DocDbProperty",
        sa.Column(
            "showOnDetail", sa.Boolean(), nullable=True, server_default="1"
        ),
        schema="core_docdb",
    )
    op.execute(
        """ UPDATE "core_docdb"."DocDbProperty" SET "showOnDetail" = true """
    )
    op.alter_column(
        "DocDbProperty",
        "showOnDetail",
        type_=sa.Boolean(),
        nullable=False,
        schema="core_docdb",
    )

    op.add_column(
        "DocDbProperty",
        sa.Column(
            "showOnSummary", sa.Boolean(), nullable=True, server_default="0"
        ),
        schema="core_docdb",
    )
    op.execute(
        """ UPDATE "core_docdb"."DocDbProperty" SET "showOnSummary" = false """
    )
    op.alter_column(
        "DocDbProperty",
        "showOnSummary",
        type_=sa.Boolean(),
        nullable=False,
        schema="core_docdb",
    )

    op.add_column(
        "DocDbProperty",
        sa.Column(
            "showOnTooltip", sa.Boolean(), nullable=True, server_default="0"
        ),
        schema="core_docdb",
    )
    op.execute(
        """ UPDATE "core_docdb"."DocDbProperty" SET "showOnTooltip" = false """
    )
    op.alter_column(
        "DocDbProperty",
        "showOnTooltip",
        type_=sa.Boolean(),
        nullable=False,
        schema="core_docdb",
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DocDbProperty", "showOnTooltip", schema="core_docdb")
    op.drop_column("DocDbProperty", "showOnSummary", schema="core_docdb")
    op.drop_column("DocDbProperty", "showOnDetail", schema="core_docdb")
    # ### end Alembic commands ###



import logging
from typing import Optional

from vortex.Tuple import Tuple

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkLoadRpcABC import (
    ACIChunkLoadRpcABC,
)
from peek_core_docdb._private.tuples.DocumentUpdateDateTuple import (
    DocumentUpdateDateTuple,
)
from peek_plugin_base.PeekVortexUtil import peekServerName, peekBackendNames
from peek_core_docdb._private.PluginNames import docDbFilt
from peek_core_docdb._private.storage.DocDbEncodedChunk import DocDbEncodedChunk
from vortex.rpc.RPC import vortexRPC

logger = logging.getLogger(__name__)


class ClientChunkLoadRpc(ACIChunkLoadRpcABC):
    def makeHandlers(self):
        """Make Handlers

        In this method we start all the RPC handlers
        start() returns an instance of it's self so we can simply yield the result
        of the start method.

        """

        yield self.loadDocumentChunks.start(funcSelf=self)
        yield self.loadDocumentIndexDelta.start(funcSelf=self)
        logger.debug("RPCs started")

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        timeoutSeconds=120,
        additionalFilt=docDbFilt,
        deferToThread=True,
    )
    def loadDocumentIndexDelta(self, indexEncodedPayload: bytes) -> bytes:
        return self.ckiChunkIndexDeltaBlocking(
            indexEncodedPayload, DocDbEncodedChunk, DocumentUpdateDateTuple
        )

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        timeoutSeconds=120,
        additionalFilt=docDbFilt,
        deferToThread=True,
    )
    def loadDocumentChunks(self, chunkKeys: list[str]) -> list[Tuple]:
        return self.ckiInitialLoadChunksPayloadBlocking(
            chunkKeys, DocDbEncodedChunk
        )



import logging

from peek_plugin_base.agent.PluginAgentEntryHookABC import (
    PluginAgentEntryHookABC,
)
from peek_core_docdb.tuples import loadPublicTuples

logger = logging.getLogger(__name__)


class AgentEntryHook(PluginAgentEntryHookABC):
    def load(self) -> None:
        # Load public tuples so they can be serialised in the agent
        loadPublicTuples()

        logger.debug("Loaded")

    def start(self):
        logger.debug("Started")

    def stop(self):
        logger.debug("Stopped")

    def unload(self):
        logger.debug("Unloaded")



import logging

from twisted.internet.defer import inlineCallbacks

from peek_core_docdb._private.worker.tasks.ImportTask import (
    createOrUpdateDocuments,
)

from peek_core_docdb._private.worker.tasks.DeleteTask import (
    deleteDocumentsForImportGropHashTask,
)

logger = logging.getLogger(__name__)


class ImportController:
    def __init__(self):
        pass

    def shutdown(self):
        pass

    @inlineCallbacks
    def createOrUpdateDocuments(self, documentsEncodedPayload: bytes):
        yield createOrUpdateDocuments.delay(documentsEncodedPayload)

    @inlineCallbacks
    def deleteDocumentsForImportGropHash(
        self, modelSetKey: str, importGroupHash: str
    ):
        yield deleteDocumentsForImportGropHashTask.delay(
            modelSetKey, importGroupHash
        )



import logging
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.PayloadEnvelope import PayloadEnvelope
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_core_docdb._private.client.controller.DocumentCacheController import (
    DocumentCacheController,
)

logger = logging.getLogger(__name__)


class ClientDocumentUpdateDateTupleProvider(TuplesProviderABC):
    def __init__(self, cacheHandler: DocumentCacheController):
        self._cacheHandler = cacheHandler

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        encodedPayload = self._cacheHandler.offlineUpdateDateTuplePayload()
        payloadEnvelope = PayloadEnvelope(filt, encodedPayload=encodedPayload)
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg
